>From 973dbe11b796395641dd3947658508ad68aebda5 Mon Sep 17 00:00:00 2001
From: Jim Nasby <Jim.Nasby@BlueTreble.com>
Date: Mon, 15 Dec 2014 18:18:40 -0600
Subject: [PATCH] Show broken rollback case
Warn if stats update doesn't happen. Add test that shows
broken stats counts when rolling back a subtrans with a truncate.

---
 src/test/regress/sql/truncate.sql | 18 ++++++++++++++++++
 1 file changed, 18 insertions(+)

diff --git a/src/test/regress/sql/truncate.sql b/src/test/regress/sql/truncate.sql
index c912345..f5a0e7a 100644
--- a/src/test/regress/sql/truncate.sql
+++ b/src/test/regress/sql/truncate.sql
@@ -251,6 +251,9 @@ begin
     perform pg_stat_clear_snapshot();
 
   end loop;
+  IF NOT updated THEN
+    RAISE WARNING 'stats update never happened';
+  END IF;
 
   TRUNCATE prevstats;  -- what a pun
   INSERT INTO prevstats SELECT newstats.*;
@@ -311,5 +314,20 @@ COMMIT;
 SELECT pg_sleep(0.5);
 SELECT * FROM wait_for_trunc_test_stats();
 
+-- now to use a savepoint: this should only count 2 inserts and have 3
+-- live tuples after commit
+BEGIN;
+INSERT INTO trunc_stats_test DEFAULT VALUES;
+INSERT INTO trunc_stats_test DEFAULT VALUES;
+SAVEPOINT p1;
+INSERT INTO trunc_stats_test DEFAULT VALUES;
+INSERT INTO trunc_stats_test DEFAULT VALUES;
+TRUNCATE trunc_stats_test;
+INSERT INTO trunc_stats_test DEFAULT VALUES;
+ROLLBACK TO SAVEPOINT p1;
+COMMIT;
+
+SELECT * FROM wait_for_trunc_test_stats();
+
 DROP TABLE prevstats CASCADE;
 DROP TABLE trunc_stats_test;
-- 
2.1.2

